<?php
/* Import from Ning BP functions */

function bp_ning_import_add_management_pages() {
	$plugin_page = add_submenu_page( 'bp-general-settings', __('Import from Ning','bp-ning-import'), __('Import from Ning','bp-ning-import'), 'manage_options', __FILE__, 'bp_ning_import_steps' );

	add_action( "admin_print_styles-$plugin_page", 'bp_ning_import_style' );
}


function bp_ning_import_style() {
	wp_enqueue_style( 'invite-anyone-admin-css', WP_PLUGIN_URL . '/import-from-ning/style.css' );
}

function bp_ning_import_steps() {

	bp_ning_import_header_markup();

	// Todo: when !isset, show an intro screen
	if ( !isset( $_POST['current_step'] ) )
		bp_ning_import_members_markup();
	else
		$current_step = $_POST['current_step'];

	switch( $current_step ) {
		case 'profiles' :
			bp_ning_import_profiles_markup();
			break;

		case 'profiles_done' :
			bp_ning_import_process_profiles();
			bp_ning_import_profile_two_markup();
			break;

		case 'groups' :
			bp_ning_import_groups_markup();
			break;

	}


?>


<?php


	echo "<pre>";
	//print_r($member_id_array);
	echo "</pre>";

}


function bp_ning_import_prepare_json( $type ) {
	$json = WP_PLUGIN_DIR . '/import-from-ning/json/ning-' . $type . '-local.json';
	$data = file_get_contents( $json );
	$data = preg_replace( '|^\(|', '', $data );
	$data = preg_replace( '|\)$|', '', $data );
	$parsed = json_decode( $data );

	return $parsed;
}

function bp_ning_import_create_user( $userdata ) {
	global $wpdb;

	$username = strtolower( preg_replace( "/\s+/", '', $userdata->fullName ) );
	$username = str_replace( '@', '', $username );
	$username = str_replace( '.', '', $username );

	// Autogenerates username by adding an integer to the end of it
	if ( username_exists( $username ) ) {
		$i = 1;
		while ( username_exists( $username . $i ) )
			$i++;
		$username = $username . $i;
	}

	// Autogenerate password
	$password = substr(md5(uniqid(microtime())), 0, 7);

	$bp_member = array(
		"user_email" => $userdata->email,
		"user_name" => $userdata->fullName,
		"already_exists" => 0
	);


	// Check for existing member
	if ( $user = get_user_by_email( $userdata->email ) ) {

		$bp_member['user_login'] = $user->user_login;
		$bp_member['id'] = $user->ID;
		$bp_member['already_exists'] = 1;

	} else {

		// create user
		$args = array(
			"user_login" => $username,
			"display_name" => $userdata->fullName,
			"nickname" => $userdata->fullName,
			"user_pass" => $password,
			"user_email" => $userdata->email
		);

		$bp_member['id'] = wp_insert_user( $args );
		$bp_member['user_login'] = $username;

		// Save the initial password to the user meta table so that the welcome email can be sent out later
		update_user_meta( $bp_member['id'], 'initial_password', $password );
	}

	// Store the Ning ID for association with content later on
	update_user_meta( $bp_member['id'], 'ning_id', $userdata->contributerName );

	return $bp_member;
}


// Step setup functions
function bp_ning_import_get_members() {
	$members = bp_ning_import_prepare_json( 'members' );
	$member_id_array = array();

	$ning_id_array = array();

	foreach ( $members as $member ) {
		 $bp_member = bp_ning_import_create_user( $member );

		 if ( isset( $bp_member['id'] ) )
		 	$member_id_array['success'][] = $bp_member;
		 else
		 	$member_id_array['error'][] = $bp_member;

		 // Create an array of Ning IDs for later reference
		 $ning_id = $member->contributorName;
		 $ning_id_array[$ning_id] = $bp_member['id'];
	}

	update_option( 'bp_ning_import_users', $member_id_array );
	update_option( 'bp_ning_user_array', $ning_id_array );

	return $member_id_array;
}

function bp_ning_import_get_profile_fields() {
	$members = bp_ning_import_prepare_json( 'members' );

	$profile_fields = array();
	$forbidden_fields = array(
		'createdDate',
		'fullName',
		'comments',
		'profilePhoto',
		'level',
		'contributorName',
		'state'
	);

	foreach ( $members as $member ) {
		$member = (array)$member;
		foreach( $member as $key => $value ) {
			if ( !in_array( $key, $profile_fields ) && !in_array( $key, $forbidden_fields) )
				$profile_fields[] = $key;
		}
	}

	return $profile_fields;
}

function bp_ning_import_process_profiles() {
	// Todo: implement avatars. It's tough because they're not all square

	if ( empty( $_POST['pf'] ) )
		return;

	$fields = $_POST['pf'];

	foreach( $fields as $key => $field ) {

		// Check to see if the user provided an alternative name for the field
		if ( $_POST['pfn'][$key] ) {
			$field = $_POST['pfn'][$key];
			$fields[$key] = $field;
		}

		// Create the field
		$args = array(
			'field_group_id' => 1,
			'name' => $field,
			'type' => 'textbox',
			'is_required' => false
			);

		if ( !xprofile_get_field_id_from_name( $field ) )
			xprofile_insert_field( $args );

	}

	// Get the field ids for the just-created fields. Todo: patch the core so that xprofile_insert_field() returns the id
	$field_ids = array();
	foreach( $fields as $field ) {
		$field_ids[$field] = xprofile_get_field_id_from_name( $field );
	}

	// Populate the new fields
	$members = bp_ning_import_prepare_json( 'members' );

	foreach( $members as $member ) {
		$member = (array)$member;
		foreach( $member as $key => $value ) {
			if ( !array_key_exists( $key, $field_ids ) )
				continue;

			$user = get_user_by_email( $member['email'] );

			if ( !xprofile_get_field_data( $field_ids[$key], $user->ID ) )
				xprofile_set_field_data( $field_ids[$key], $user->ID, $value );
		}
	}

}

function bp_ning_import_get_discussions() {
	$ning_id_array = get_option( 'bp_ning_user_array' );

	$discussions = bp_ning_import_prepare_json( 'discussions' );

	// Get list of Ning groups for cross reference
	$groups = bp_ning_import_prepare_json( 'groups' );

	if ( !$ning_group_id_array = get_option( 'bp_ning_group_array' ) )
		$ning_group_id_array = array();

	foreach ( $groups as $group ) {
		// Create the group
		$ning_group_creator_id = $group->contributorName;
		$creator_id = $ning_id_array[$ning_group_creator_id];

		$status = ( $group->groupPrivacy == 'private' ) ? 'private' : 'public';

		$ndate = strtotime( $group->createdDate );

		$date_created = date( "Y-m-d H:i:s", $ndate );

		$slug = sanitize_title( esc_attr( $group->title ) );

		$args = array(
			'creator_id' => $creator_id,
			'name' => $group->title,
			'description' => $group->description,
			'slug' => $slug,
			'status' => $status,
			'enable_forum' => 1,
			'date_created' => $date_created
		);

		if ( !BP_Groups_Group::group_exists( $slug ) ) {
			if ( $group_id = groups_create_group( $args ) ) {
				groups_update_groupmeta( $group_id, 'last_activity', $date_created );
				groups_update_groupmeta( $group_id, 'total_member_count', 1 );

				groups_new_group_forum( $group_id, $group->title, $group->description );

				$ning_group_id = $group->id;
				$ning_group_id_array[$ning_group_id] = $group_id;
			}
		}
	}

	// Loop through each discussion. If the topic doesn't have a corresponding group, create one. Then insert the forum items.

	foreach ( $discussions as $discussion ) {

		// If the discussion originally came from a category, it'll need to go into a group.
		if ( isset( $discussion->category ) ) {
			$slug = sanitize_title( esc_attr( $discussion->category ) );

			$ning_group_creator_id = $discussion->contributorName;
			$creator_id = $ning_id_array[$ning_group_creator_id];

			$ndate = strtotime( $discussion->createdDate );
			$date_created = date( "Y-m-d H:i:s", $ndate );

			if ( !$group_id = BP_Groups_Group::group_exists( $slug ) ) {

				$args = array(
					'creator_id' => $creator_id,
					'name' => $discussion->category,
					'description' => $discussion->category,
					'slug' => groups_check_slug( $slug ),
					'status' => 'public',
					'enable_forum' => 1,
					'date_created' => $date_created
				);

				if ( $group_id = groups_create_group( $args ) ) {
					groups_update_groupmeta( $group_id, 'last_activity', $date_created );
					groups_update_groupmeta( $group_id, 'total_member_count', 1 );

					groups_new_group_forum( $group_id, $discussion->category, $discussion->category );

					$ning_group_id = $discussion->id;
					$ning_group_id_array[$ning_group_id] = $group_id;
				}
			}
echo "<pre>"; print_r($discussion); echo "</pre>";
			// The group has now been created, so we can handle the discussion itself

			$args = array(
				'topic_title' => $discussion->title,
				'topic_slug' => groups_check_slug( sanitize_title( esc_attr( $discussion->title ) ) ),
				'topic_text' => $discussion->description,
				'topic_poster' => $creator_id,
				'topic_poster_name' => bp_core_get_user_displayname( $creator_id ),
				'topic_last_poster' => $creator_id,
				'topic_last_poster_name' => bp_core_get_user_displayname( $creator_id ),
				'topic_start_time' => $date_created,
				'topic_time' => $date_created,
				'forum_id' => groups_get_groupmeta( $group_id, 'forum_id' )
			);

			$topic_id = bp_forums_new_topic( $args );

			$topic = bp_forums_get_topic_details( $topic_id );
			$group = new BP_Groups_Group( $group_id );

			// Activity item
			$activity_action = sprintf( __( '%s started the forum topic %s in the group %s:', 'buddypress'), bp_core_get_userlink( $creator_id ), '<a href="' . bp_get_group_permalink( $group ) . 'forum/topic/' . $topic->topic_slug .'/">' . attribute_escape( $topic->topic_title ) . '</a>', '<a href="' . bp_get_group_permalink( $group ) . '">' . attribute_escape( $group->name ) . '</a>' );
			$activity_content = bp_create_excerpt( $topic_text );

			/* Record this in activity streams */
			groups_record_activity( array(
				'user_id' => $creator_id,
				'action' => apply_filters( 'groups_activity_new_forum_topic_action', $activity_action, $discussion->description, &$topic ),
				'content' => apply_filters( 'groups_activity_new_forum_topic_content', $activity_content, $discussion->description, &$topic ),
				'primary_link' => apply_filters( 'groups_activity_new_forum_topic_primary_link', bp_get_group_permalink( $group ) . 'forum/topic/' . $topic->topic_slug . '/' ),
				'type' => 'new_forum_topic',
				'item_id' => $group_id,
				'secondary_item_id' => $topic->topic_id,
				'hide_sitewide' => 0
			) );

			do_action( 'groups_new_forum_topic', $group_id, &$topic );







			// Now check for comments

			if ( isset( $discussion->comments ) ) {
				foreach ( $discussion->comments as $reply ) {
					$ning_group_creator_id = $reply->contributorName;
					$creator_id = $ning_id_array[$ning_group_creator_id];

					$ndate = strtotime( $reply->createdDate );
					$date_created = date( "Y-m-d H:i:s", $ndate );

					$args = array(
						'topic_id' => $topic_id,
						'post_text' => $reply->description,
						'post_time' => $date_created,
						'poster_id' => $creator_id,
						'poster_ip' => '192.168.1.1'
					);

					$post_id = bp_forums_insert_post( $args );

					// Activity item
					$topic = bp_forums_get_topic_details( $topic_id );

					$activity_action = sprintf( __( '%s posted on the forum topic %s in the group %s:', 'buddypress'), bp_core_get_userlink( $creator_id ), '<a href="' . bp_get_group_permalink( $group_id ) . 'forum/topic/' . $topic->topic_slug .'/">' . attribute_escape( $topic->topic_title ) . '</a>', '<a href="' . bp_get_group_permalink( $group ) . '">' . attribute_escape( $group->name ) . '</a>' );
					$activity_content = bp_create_excerpt( $reply->description );
					$primary_link = bp_get_group_permalink( $group ) . 'forum/topic/' . $topic->topic_slug . '/';

					if ( $page )
						$primary_link .= "?topic_page=" . $page;
					//echo $primary_link; die();

					groups_record_activity( array(
						'user_id' => $creator_id,
						'action' => apply_filters( 'groups_activity_new_forum_post_action', $activity_action, $post_id, $reply->description, &$topic ),
						'content' => apply_filters( 'groups_activity_new_forum_post_content', $activity_content, $post_id, $reply->description, &$topic ),
						'primary_link' => apply_filters( 'groups_activity_new_forum_post_primary_link', "{$primary_link}#post-{$post_id}" ),
						'type' => 'new_forum_post',
						'item_id' => $group_id,
						'secondary_item_id' => $post_id,
						'hide_sitewide' => 0
					) );

					do_action( 'groups_new_forum_topic_post', $group_id, $post_id );




				}
			}

		}
	}

	print_r($ning_group_id_array);

	update_option( 'bp_ning_group_array', $ning_group_id_array );


echo "<pre>";
//print_r($discussions);die();

	foreach ( $members as $member ) {
		$member = (array)$member;
		foreach( $member as $key => $value ) {
			if ( !in_array( $key, $profile_fields ) && !in_array( $key, $forbidden_fields) )
				$profile_fields[] = $key;
		}
	}

	return $profile_fields;
}


// Markup functions
function bp_ning_import_header_markup() {
?>
<div class="wrap">

	<h2><?php _e( 'Import Users from Ning', 'bp-ning-import' ) ?></h2>
<?php
}

function bp_ning_import_members_markup() {
	$member_id_array = bp_ning_import_get_members();
?>
	<?php if ( !empty( $member_id_array['success'] ) ) : ?>
		<h3><?php _e( 'Accounts created', 'bp-ning-import' ) ?></h3>

		<p><?php _e( 'The following members have either been found in your system, or have had accounts created for them.', 'bp-ning-import' ) ?></p>

		<p><?php _e( 'You will have a chance later on to email new members with their login information.', 'bp-ning-import' ) ?></p>


		<div class="submit">
			<form method="post" action="">
				<input class="button primary-button" type="submit" id='submit' name='submit' value="<?php _e( 'Continue' ) ?>">
				<input type="hidden" id="current_step" name="current_step" value="profiles">
			</form>
		</div>

		<table id="ning-import-users">

		<tr>
			<th><?php _e( 'User ID' ) ?></th>
			<th><?php _e( 'Name' ) ?></th>
			<th><?php _e( 'Login' ) ?></th>
			<th><?php _e( 'Email address' ) ?></th>
		</tr>


		<?php foreach( $member_id_array['success'] as $bp_member ) : ?>
			<tr>
				<td><?php echo $bp_member['id'] ?></td>
				<td><?php echo $bp_member['user_name'] ?></td>
				<td><?php echo $bp_member['user_login'] ?></td>
				<td><?php echo $bp_member['user_email'] ?></td>
			</tr>
		<?php endforeach; ?>
		</table>
	<?php else : ?>
		<h3><?php _e( 'No accounts created', 'bp-ning-import' ) ?></h3>

		<p><?php _e( 'Sorry, I was unable to create any accounts.', 'bp-ning-import' ) ?></p>

	<?php endif; ?>
<?
}

function bp_ning_import_profiles_markup() {
	$profile_fields = bp_ning_import_get_profile_fields();
?>
	<form method="post" action="">

	<?php if ( !empty( $profile_fields ) ) : ?>
		<h3><?php _e( 'Profile fields', 'bp-ning-import' ) ?></h3>

		<p><?php _e( 'The following profile fields were identified in your Ning data. Select the ones you\'d like to keep as BuddyPress profile fields. Your members\' data will be imported automatically.', 'bp-ning-import' ) ?></p>

		<p><?php _e( 'You can also edit or add profile fields later on at Dashboard > BuddyPress > Profile Field Setup.', 'bp-ning-import' ) ?></p>

		<table id="ning-import-profile-fields">

		<tr>
			<th> </th>
			<th><?php _e( 'Original field name' ) ?></th>
			<th><?php _e( 'New field name (optional)' ) ?></th>
		</tr>

		<?php foreach( $profile_fields as $pf ) : ?>
			<?php if ( xprofile_get_field_id_from_name( $pf ) ) continue; ?>
			<tr>
				<td> <input type="checkbox" name="pf[]" value="<?php echo $pf ?>" checked> </td>
				<td><?php echo $pf ?></td>
				<td><input type="text" name="pfn[]" /></td>
			</tr>
		<?php endforeach; ?>

		</table>

	<?php else : ?>
		<h3><?php _e( 'Profile fields', 'bp-ning-import' ) ?></h3>

		<p><?php _e( 'No additional profile fields were found.', 'bp-ning-import' ) ?></p>

	<?php endif; ?>

	<div class="submit">
			<input class="button primary-button" type="submit" id='submit' name='submit' value="<?php _e( 'Continue' ) ?>">
			<input type="hidden" id="current_step" name="current_step" value="profiles_done" />
	</div>

	</form>

<?
}

function bp_ning_import_profile_two_markup() {
	$profile_fields = bp_ning_import_get_profile_fields();
?>
	<h3><?php _e( 'Profile fields', 'bp-ning-import' ) ?></h3>

	<p><?php _e( 'Profile data successfully imported! Click Continue to continue the import process.', 'bp-ning-import' ) ?></p>

	<div class="submit">
		<form method="post" action="">
			<input class="button primary-button" type="submit" id='submit' name='submit' value="<?php _e( 'Continue' ) ?>">
			<input type="hidden" id="current_step" name="current_step" value="groups" />
		</form>
	</div>

<?
}

function bp_ning_import_groups_markup() {
	$discussions = bp_ning_import_get_discussions();
	echo "groups"; die();
?>
	<form method="post" action="">

	<?php if ( !empty( $profile_fields ) ) : ?>
		<h3><?php _e( 'Profile fields', 'bp-ning-import' ) ?></h3>

		<p><?php _e( 'The following profile fields were identified in your Ning data. Select the ones you\'d like to keep as BuddyPress profile fields. Your members\' data will be imported automatically.', 'bp-ning-import' ) ?></p>

		<p><?php _e( 'You can also edit or add profile fields later on at Dashboard > BuddyPress > Profile Field Setup.', 'bp-ning-import' ) ?></p>

		<table id="ning-import-profile-fields">

		<tr>
			<th> </th>
			<th><?php _e( 'Original field name' ) ?></th>
			<th><?php _e( 'New field name (optional)' ) ?></th>
		</tr>

		<?php foreach( $profile_fields as $pf ) : ?>
			<?php if ( xprofile_get_field_id_from_name( $pf ) ) continue; ?>
			<tr>
				<td> <input type="checkbox" name="pf[]" value="<?php echo $pf ?>" checked> </td>
				<td><?php echo $pf ?></td>
				<td><input type="text" name="pfn[]" /></td>
			</tr>
		<?php endforeach; ?>

		</table>

	<?php else : ?>
		<h3><?php _e( 'Profile fields', 'bp-ning-import' ) ?></h3>

		<p><?php _e( 'No additional profile fields were found.', 'bp-ning-import' ) ?></p>

	<?php endif; ?>

	<div class="submit">
			<input class="button primary-button" type="submit" id='submit' name='submit' value="<?php _e( 'Continue' ) ?>">
			<input type="hidden" id="current_step" name="current_step" value="profiles_done" />
	</div>

	</form>

<?
}



function bp_ning_import_process_profile_fields( $data ) {

	$u_keys = $data['u_keys'];
	$u_data = $data['u_data'];

	$is_fields = 0;
	foreach ( $u_keys as $k ) {
		if ( $u_key != 'Name' && $u_key != 'Email' && $u_key != 'Profile Address' && $u_key != 'Date Joined' )
			$is_fields = 1;
	}

	if ( !$is_fields )
		return;

?>
	<div class="wrap">
	<h2><?php _e( 'Import Users from Ning', 'bp-ning-import' ) ?></h2>

	<h3><?php _e( 'Step Three: Import Additional Profile Data to BuddyPress', 'bp-ning-import' ) ?></h3>

	<p><?php _e( "Several additional profile fields have been found in the Ning data. If you'd like, these fields can be imported as BuddyPress profile fields.", 'bp-ning-import' ) ?></p>

	<form action='<?php echo $_SERVER["REQUEST_URI"]; ?>' method="post">
	<ul>
	<?php foreach ( $u_keys as $k ) : ?>
		<?php if ( $k != 'Name' && $k != 'Profile Address' && $k != 'Date Joined' ) : ?>
			<?php /* Don't need to create a field if it already exists */ ?>
			<?php if ( xprofile_get_field_id_from_name( $u_key ) ) continue; ?>

			<li><input type="checkbox" name="fields[]" id="fields[]" value="<?php echo $k ?>"  /> <?php echo $k ?></li>

		<?php endif; ?>
	<?php endforeach; ?>
	</ul>


	<p><?php _e( "Check the fields you'd like to import and click <strong>Continue</strong>. (You can edit these fields later at <strong>Dashboard > BuddyPress > Profile Field Setup</strong>.)", 'bp-ning-import' ) ?></p>

	<?php /* Must send the $u_keys as well in order to key data correctly in the final step */ ?>
	<?php foreach( $u_keys as $k ) : ?>
		<input type="hidden" name="u_keys[]" id="u_keys[]" value="<?php echo esc_attr($k) ?>" />
	<?php endforeach; ?>

	<?php foreach( $u_data as $user_id => $u ) : ?>
		<input type="hidden" name="u_data[<?php echo $user_id ?>]" id="u_data[<?php echo $user_id ?>]" value="<?php echo $u ?>" />
	<?php endforeach; ?>

	<input type="hidden" name="step_three" id="step_three" value="1" />

	<div class="submit">
		<input class="button primary-button" type="submit" name="info_update" value="<?php _e( 'Continue', 'bp-ning-import' ); ?> &raquo;" />
	</div>

	</form>

	</div>
<?php
}
add_action( 'ning_import_step_two_submit', 'bp_ning_import_process_profile_fields' );

function bp_ning_import_build_profile_fields( $data ) {
	global $bp;

	if ( !is_array( $data['fields'] ) )
		return;

	//print "<pre>";
	//print_r($data); die();

	/* Create the fields */
	$fields = array();
	foreach ( $data['fields'] as $field ) {
		$args = array(
			'field_group_id' => 1,
			'name' => $field,
			'type' => 'textbox',
			'is_required' => false
			);

		if ( !xprofile_get_field_id_from_name( $field ) )
			xprofile_insert_field( $args );

		$key = array_search( $field, $data['u_keys'] );
		$fields[$key] = $field;
	}

	/* Loop through the user data to populate the fields */
	foreach ( $data['u_data'] as $user_id => $u ) {
		$u = explode( '|', $u );

		/* Now loop through each of the fields */
		foreach( $fields as $key => $field ) {
			if ( !xprofile_get_field_data( $field, $user_id ) )
				xprofile_set_field_data( $field, $user_id, $u[$key] );
		}
	}

	$name_key = array_search( 'Name', $data['u_keys'] );

?>
	<div class="wrap">
	<h2><?php _e( 'Import Users from Ning', 'bp-ning-import' ) ?></h2>

	<h3><?php _e( 'Success!', 'bp-ning-import' ) ?></h3>

	<p><?php _e( "Success! Your users and their profile data have been successfully imported into BuddyPress.", 'bp-ning-import' ) ?></p>

	<p><?php _e( "Click on the links below to visit the new profiles that have been generated for your users. ", 'bp-ning-import' ) ?></p>

	<div class="bp-ning-import-success-users">
		<ul>
		<?php foreach ( $data['u_data'] as $user_id => $u ) : ?>
			<?php $u = explode( '|', $u ) ?>
			<li><a href="<?php echo bp_core_get_user_domain( $user_id ) .  $bp->profile->slug ?>"><?php echo $u[$name_key] ?></a></li>

		<?php endforeach; ?>
		</ul>
	</div>

	<p><?php _e( "Your users can customize their own profiles by navigating to <strong>My Account > Profile > Edit Profile</strong>. As the administrator, you can manage profile fields at <strong>Dashboard > BuddyPress > Profile Field Setup</strong>", 'bp-ning-import' ) ?></p>

	<p><?php _e( "Enjoy using BuddyPress!", 'bp-ning-import' ) ?></p>

	<?php bp_ning_import_donate_message(); ?>

<?php
}

?>